# -*- coding: utf-8 -*-

"""
 (c) 2018 - Copyright Red Hat Inc

 Authors:  The Dream Team
   Pierre-Yves Chibon <pingou@pingoured.fr>
   Ralph Bean <rbean@redhat.com>

"""
from __future__ import print_function

import logging
import os

import werkzeug

import pagure.config
from pagure.lib import model
from pagure.lib.git_auth import Gitolite3Auth, _read_file


if 'PAGURE_CONFIG' not in os.environ \
        and os.path.exists('/etc/pagure/pagure.cfg'):
    os.environ['PAGURE_CONFIG'] = '/etc/pagure/pagure.cfg'


_log = logging.getLogger(__name__)
_config = pagure.config.reload_config()


_blacklist = '''  -    f[0-9]+.* = @all
  -    epel[0-9]+.* = @all
  -    el[0-9] = @all
  -    olpc[0-9] = @all
  -    c[0-9]+.*'''


class CentOSAuth(Gitolite3Auth):
    """ A CentOS' gitolite authentication module. """

    @classmethod
    def _process_project(cls, project, config, global_pr_only):
        """ Generate the gitolite configuration for the specified project.

        :arg project: the project to generate the configuration for
        :type project: pagure.lib.model.Project
        :arg config: a list containing the different lines of the
            configuration file
        :type config: list
        :arg global_pr_only: boolean on whether the pagure instance enforces
            the PR workflow only or not
        :type global_pr_only: bool
        :return: the updated config
        :return type: list

        """

        _log.debug('    Processing project: %s', project.fullname)

        # Check if the project or the pagure instance enforce the PR
        # only development model.
        pr_only = project.settings.get('pull_request_access_only', False)

        for repos in ['repos', 'requests/']:
            if repos == 'repos':
                config.append('repo %s' % project.fullname)

                # Add code here if you want all-project, all-branch access
                # for a certain group
                # config.append('  RW  = releng')

                # Do not grant access to project enforcing the PR model
                if pr_only or (global_pr_only and not project.is_fork):
                    continue
                repos = ''
            else:
                config.append('repo %s%s' % (repos, project.fullname))

            if repos not in ['tickets/', 'requests/']:
                config.append('  R   = @all')

            access = 'RWC'
            if project.is_fork:
                access = 'RW+C'

            users = sorted(set([project.user.user]).union(
                set(
                    user.user
                    for user in project.committers
                    if user != project.user)))
            groups = sorted(set(
                group.group_name
                for group in project.committer_groups
            ))

            if repos == '' and not project.is_fork:

                if project.namespace == 'rpms':
                    for branch in _config.get('SUPPORTED_BRANCHES', []):
                        for sig in _config.get('SUPPORTED_SIG', []):
                                config.append('  %s %s-%s-* = @%s' % (
                                    access, branch, sig, sig
                                    )
                                )
                    pnt_access = []
                    for branch in _config.get(
                            'PNT_BRANCHES', _config.get('SUPPORTED_BRANCHES', [])):
                        pnt_access.append('  RWC %s = @pnt' % branch)
                    config.extend(pnt_access)
                    # Then, blacklist a pattern over that (after).
                    config.append(_blacklist)

                else:
                    config.append('  %s = %s' % (
                        access, ' '.join(users)
                        )
                    )
                    if groups:
                        config.append('  %s = @%s' % (
                            access, ' @'.join(groups)
                            )
                        )

            if project.is_fork or repos in ['tickets/', 'requests/']:
                # Grant all branch access on forks only
                if groups:
                    config.append('  %s = @%s' % (access, ' @'.join(groups)))

                config.append('  %s = %s' % (access, ' '.join(users)))

            if project.is_fork:
                # Add deploykeys but only on fork for CentOS
                for deploykey in project.deploykeys:
                    access = 'R'
                    if deploykey.pushaccess:
                        access = 'RW'
                        if project.is_fork:
                            access = 'RW+C'
                    # Note: the replace of / with _ is because gitolite
                    # users can't contain a /. At first, this might look
                    # like deploy keys in a project called
                    # $namespace_$project would give access to the repos of
                    # a project $namespace/$project or vica versa, however
                    # this is NOT the case because we add the deploykey.id
                    # to the end of the deploykey name, which means it is
                    # unique. The project name is solely there to make it
                    # easier to determine what project created the deploykey
                    # for admins.
                    config.append('  %s = deploykey_%s_%s' %
                                  (access,
                                   werkzeug.secure_filename(project.fullname),
                                   deploykey.id))

            config.append('')

        return config
